#include <libmp/all.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "../util.h"
#include "../intl.h"
#include "../cmd.h"
#include "../log.h"

enum field {
  FIELD_NAME = 0,
  FIELD_DESC = 1,
  FIELD_VERSION = 2,
};

bool contains(char *string, char *search){
  if(NULL == string || NULL == search)
    return false;

  size_t string_size = strlen(string);
  size_t search_size = strlen(search);
  size_t i = 0;

  char string_lower[string_size+1];
  char search_lower[search_size+1];

  for(i = 0; i < string_size; i++)
    string_lower[i] = tolower(string[i]);
  string_lower[i] = 0;
  
  for(i = 0; i < search_size; i++)
    search_lower[i] = tolower(search[i]);
  search_lower[i] = 0;

  return strstr(string_lower, search_lower) != NULL;
}

bool cmd_search_database(lm_ctx_t *ctx, args_t *args, char *query, enum field field) {
  lm_entry_t *list = malloc(sizeof(lm_entry_t));
  char *selected = NULL;
  size_t indx = 0, i = 0;
  bool ret = false;

  while(lm_ctx_database_next(ctx, &list[indx])){
    selected = NULL;

    switch (field) {
    case FIELD_NAME:
      selected = list[indx].name;
      break;

    case FIELD_DESC:
      selected = list[indx].desc;
      break;

    case FIELD_VERSION:
      selected = list[indx].version;
      break;
    }

    if(!contains(selected, query))
      continue;

    list = realloc(list, sizeof(lm_entry_t)*((++indx)+1));
    lm_entry_init(&list[indx]);
  }

  if(indx <= 0){
    if(!args_get_bool(args, "grep"))
      info(_("No results, maybe you forgot to sync?"));
    goto end;
  }

  if(!args_get_bool(args, "grep"))
    info(indx > 1 ? _("Found "FG_BOLD"%d"FG_RESET" results") : _("Found "FG_BOLD"%d"FG_RESET" result"), indx);

  for(i = 0; i < indx; i++){
    if(args_get_bool(args, "grep")){
      if(args_get_bool(args, "desc"))
        printf("%s:%s:%s\n", list[i].name, list[i].version, list[i].desc);
      else
        printf("%s:%s\n", list[i].name, list[i].version);
      continue;
    }

    if(args_get_bool(args, "desc"))
      printf(FG_BOLD"%s"FG_RESET"_"FG_BOLD FG_GREEN"%s"FG_RESET": %s\n", list[i].name, list[i].version, list[i].desc);
    else
      printf(FG_BOLD"%s"FG_RESET"_"FG_BOLD FG_GREEN"%s"FG_RESET"\n", list[i].name, list[i].version);
  }

  ret = true;
end:
  for(i = 0; i < (indx+1); i++)
    lm_ctx_database_next_free(ctx, &list[indx]);
  free(list);
  return ret;
}

bool cmd_search_pools(lm_ctx_t *ctx, args_t *args, char *query, enum field field) {
  lm_pkg_t **list = malloc(sizeof(lm_pkg_t*));
  char *selected = NULL;
  size_t indx = 0, i = 0;
  lm_ctx_list_t plist;
  bool ret = false;

  list[indx] = NULL;
    
  if(!lm_ctx_sync(ctx, false, NULL, NULL)){
    if(!args_get_bool(args, "grep"))
      info(_("Failed to load package pools: %s"), lm_strerror());
    goto end;
  }

  if(!lm_ctx_list(ctx, &plist)){
    if(!args_get_bool(args, "grep"))
      info(_("Failed to get package list: %s"), lm_strerror());
    goto end;
  }

  while((list[indx] = lm_ctx_list_next(&plist)) != NULL){
    selected = NULL;

    switch (field) {
    case FIELD_NAME:
      selected = list[indx]->data.name;
      break;

    case FIELD_DESC:
      selected = list[indx]->data.desc;
      break;

    case FIELD_VERSION:
      selected = list[indx]->data.version;
      break;
    }

    if(!contains(selected, query))
      continue;

    list = realloc(list, sizeof(lm_pkg_t*)*((++indx)+1));
    list[indx] = NULL;
  }

  if(indx <= 0){
    if(!args_get_bool(args, "grep"))
      info(_("No results, maybe you forgot to sync?"));
    goto end;
  }

  if(!args_get_bool(args, "grep"))
    info(indx > 1 ? _("Found "FG_BOLD"%d"FG_RESET" results") : _("Found "FG_BOLD"%d"FG_RESET" result"), indx);

  for(i = 0; i < indx; i++){
    if(args_get_bool(args, "grep")){
      if(args_get_bool(args, "desc"))
        printf("%s:%s:%s\n", list[i]->data.name, list[i]->data.version, list[i]->data.desc);
      else
        printf("%s:%s\n", list[i]->data.name, list[i]->data.version);
      continue;
    }

    if(args_get_bool(args, "desc"))
      printf(FG_BOLD"%s"FG_RESET"_"FG_BOLD FG_GREEN"%s"FG_RESET": %s\n",
             list[i]->data.name, list[i]->data.version, list[i]->data.desc);
    else
      printf(FG_BOLD"%s"FG_RESET"_"FG_BOLD FG_GREEN"%s"FG_RESET"\n",
             list[i]->data.name, list[i]->data.version);
  }

end:
  lm_ctx_list_free(&plist);
  free(list);
  return ret;
}

bool cmd_search(lm_ctx_t *ctx, config_t *config, args_t *args){
  char *arg = NULL;

  args_split(args, "grep");
  args_split(args, "desc");
  args_split(args, "database");

  args_register(args, "grep");
  args_register(args, "desc");
  args_register(args, "field");
  args_register(args, "database");

  if((arg = args_register_check(args)) != NULL){
    error(_("Unknown argument: "FG_BOLD"--%s"FG_RESET", run with "FG_BOLD"--help"FG_RESET" to list the avaliable arguments"), arg);
    return false;
  }

  if(args_get_bool(args, "help")){
    info(_("Usage: " FG_BOLD "search <options> [query]"));
    info(_("Listing options for the list command:"));
    printf(_("    "FG_BOLD"--grep"FG_RESET":\tmakes the output \"grepable\"\n"));
    printf(_("    "FG_BOLD"--desc"FG_RESET":\tshow package description of the results\n"));
    printf(_("    "FG_BOLD"--field"FG_RESET":\tpackage field to search by (\"name\", \"desc\" or \"version\")\n"));
    printf(_("    "FG_BOLD"--database"FG_RESET":\tsearch the database instead of the pools\n"));
    return true;
  }

  enum field field = FIELD_NAME;
  char *query = NULL, *field_val = NULL;

  for(size_t i = 1; i < args->count; i++){
    if(NULL != args->list[i].name)
      continue;

    if(NULL == query && NULL != (query = args->list[i].value))
      continue;

    else if(NULL != query && NULL != (query = args->list[i].value)){
      if(!args_get_bool(args, "grep"))
        error(_("Please specify only a single query for searching"));
      return false;
    }
  }

  if(NULL == query){
    if(!args_get_bool(args, "grep"))
      error(_("Please specify a query for searching"));
    return false;
  }

  if(NULL == (field_val = args_get_string(args, "field")))
    field = FIELD_NAME;
  else if(eq(field_val, "desc"))
    field = FIELD_DESC;
  else if(eq(field_val, "version"))
    field = FIELD_VERSION;

  if(args_get_bool(args, "database"))
    return cmd_search_database(ctx, args, query, field);
  return cmd_search_pools(ctx, args, query, field);
}

